package ovh.herisson.Clyde.EndPoints.ScientificPublications;

/******************************************************
 * @file ResearchController.java
 * @author Bartha Maxime
 * @scope Publications Scientifiques
 *
 * API class for the researches
 ******************************************************/

import lombok.AllArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import ovh.herisson.Clyde.DTO.ScientificPublications.ResearchDTO;
import ovh.herisson.Clyde.DTO.ScientificPublications.ResearcherDTO;
import ovh.herisson.Clyde.Responses.UnauthorizedResponse;
import ovh.herisson.Clyde.Services.AuthenticatorService;
import ovh.herisson.Clyde.Services.ScientificPublications.ResearchesService;
import ovh.herisson.Clyde.Tables.Role;
import ovh.herisson.Clyde.Tables.ScientificPublications.Research;
import ovh.herisson.Clyde.Tables.ScientificPublications.Researcher;

import java.util.ArrayList;
import java.util.Map;

@RestController
@CrossOrigin(originPatterns = "*", allowCredentials = "true")
@AllArgsConstructor
public class ResearchController {



   private final ResearchesService researchesServ;

   private final AuthenticatorService authServ;

    /** Is accessible by anyone
     * but if the user doesn't have the permission to download the research
     * the return Research Download URL will be null
     */
    @GetMapping("/research/{id}")
    public ResponseEntity<ResearchDTO> getResearch(@RequestHeader(value = "Authorization", required = false) String token, @PathVariable long id){

        Research research = researchesServ.getResearchById(id);

        if (research == null)
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);

        if (researchesServ.hasNoAccessTo(research,authServ.getUserFromToken(token))){
           research.setPdfLocation(null);
        }// If the user doesn't have access to the document he can't download the pdf

        return new ResponseEntity<>(ResearchDTO.construct(research), HttpStatus.OK);
    }

    /**
     * @param token  used to know if the user can download or not the research pdf
     * @return every research
     */
    @GetMapping("/researches")
    public ResponseEntity<Iterable<ResearchDTO>> getResearches(@RequestHeader(value = "Authorization",required = false) String token){
        Iterable<Research> researches = researchesServ.getAllResearches();

        ArrayList<ResearchDTO> toReturnResearches = new ArrayList<>();

        for (Research research: researches){
            if (researchesServ.hasNoAccessTo(research,authServ.getUserFromToken(token))){
                research.setPdfLocation(null);
            }
            toReturnResearches.add(ResearchDTO.construct(research));
        }
        return new ResponseEntity<>(toReturnResearches,HttpStatus.OK);
    }


    @GetMapping("/researches/{id}")
    public ResponseEntity<Iterable<ResearchDTO>> getResearchesFromResearcher(@RequestHeader(value = "Authorization",required = false) String token,
                                                                             @PathVariable Long id
    ){
        Iterable<Research> researches = researchesServ.getResearchesByAuthor(id);
        if (researches == null) return new ResponseEntity<>(HttpStatus.NOT_FOUND);

        ArrayList<ResearchDTO> toReturnResearches = new ArrayList<>();

        for (Research research: researches){
            if (researchesServ.hasNoAccessTo(research,authServ.getUserFromToken(token))){
                research.setPdfLocation(null);
            }
            toReturnResearches.add(ResearchDTO.construct(research));
        }
        return new ResponseEntity<>(toReturnResearches,HttpStatus.OK);
    }

    /** post a new research
     *
     * @return the research saved
     */
    @PostMapping("/research")
    public ResponseEntity<Research> postResearch(@RequestHeader("Authorization") String token, @RequestBody Research research){

        if (authServ.isNotIn(new Role[]{Role.Admin},token) &&
                researchesServ.getResearcherByUser(authServ.getUserFromToken(token)) == null){
            return new UnauthorizedResponse<>(null);
        } // if the poster isn't a researcher

        return new ResponseEntity<>(researchesServ.saveResearch(research), HttpStatus.OK);
    }

    /** post updates to the research
     *
     */
    @PatchMapping("/research/{id}")
    public ResponseEntity<String> patchResearch(@RequestHeader("Authorization") String token,
                                          @RequestBody Map<String,Object> updates,
                                          @PathVariable long id
                                          )
    {
        Research research = researchesServ.getResearchById(id);
        Researcher researcher = researchesServ.getResearcherByUser(authServ.getUserFromToken(token));

        if (research == null)
            return new ResponseEntity<>(HttpStatus.NOT_FOUND);

        if (authServ.isNotIn(new Role[]{Role.Admin,Role.Secretary},token) &&
                researcher != research.getAuthor()) {
            return new UnauthorizedResponse<>(null);
        }

      researchesServ.modifyResearchData(research, updates);
      return new ResponseEntity<>(HttpStatus.OK);
    }

    /** Only Admin, Secretary and author can delete a research
     *
     */
    @DeleteMapping("/research/{id}")
    public ResponseEntity<String> deleteResearch(@RequestHeader("Authorization") String token, @PathVariable long id){

        Research research = researchesServ.getResearchById(id);
        Researcher researcher = researchesServ.getResearcherByUser(authServ.getUserFromToken(token));

        if (research == null)
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        if (authServ.isNotIn(new Role[]{Role.Admin, Role.Secretary},token) &&
                researcher != research.getAuthor()){
            return new UnauthorizedResponse<>(null);
        }

        researchesServ.deleteResearch(research);
        return new ResponseEntity<>(HttpStatus.OK);
    }


    /////////////
    // Co-authors Part

    /** get all the co-authors in a research
     *
     * @param id the research id
     * @return all the co-authors associated with the research
     */
    @GetMapping("/research/{id}/co-authors")
    public  ResponseEntity<Iterable<ResearcherDTO>> getCoAuthors(@PathVariable long id){
       Research research = researchesServ.getResearchById(id);

       if (research == null) return new ResponseEntity<>(HttpStatus.NOT_FOUND);

       Iterable<Researcher> researchers= researchesServ.getCoAuthors(research);
       ArrayList<ResearcherDTO> toReturn = new ArrayList<>();

       for (Researcher researcher: researchers){
           toReturn.add(ResearcherDTO.construct(researcher));
       }
       return new ResponseEntity<>(toReturn,HttpStatus.OK);
    }

    /** post co-authors to the research
     *
     * @param token the Authorization header
     * @param researchersId the co-authors ids
     * @param id the research id
     * @return the added researchers
     */
    @PostMapping("/research/{id}/co-authors")
    public ResponseEntity<Iterable<ResearcherDTO>> postCoAuthor(@RequestHeader("Authorization") String token,
                                                              @RequestBody Iterable<Long> researchersId,
                                                              @PathVariable long id)
    {
        Research research = researchesServ.getResearchById(id);
        if (research == null) return new ResponseEntity<>(HttpStatus.NOT_FOUND);

        if (authServ.isNotIn(new Role[]{Role.Admin}, token) || research.getAuthor() != researchesServ.getResearcherByUser(authServ.getUserFromToken(token)))
            return new UnauthorizedResponse<>(null);

        if (!researchesServ.saveCoAuthors(researchersId,research))
            return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        ArrayList<ResearcherDTO> toReturn = new ArrayList<>();
        for (Long reId: researchersId){
            toReturn.add(ResearcherDTO.construct(researchesServ.getResearcherById(reId)));
        }

        return  new ResponseEntity<>(toReturn,HttpStatus.OK);
    }

    /** deletes the co-author with *coAuthorId* from the research with the *researchId*
     */
    @DeleteMapping("/research/{researchId}/co-author/{coAuthorId}")
    public ResponseEntity<String> deleteCoAuthor(@RequestHeader("Authorization") String token,
                                                 @PathVariable long researchId,
                                                 @PathVariable long coAuthorId)
    {
        Research research = researchesServ.getResearchById(researchId);
        Researcher coAuthor = researchesServ.getResearcherById(coAuthorId);

        if (authServ.isNotIn(new Role[]{Role.Admin}, token) || research.getAuthor() != researchesServ.getResearcherByUser(authServ.getUserFromToken(token)))
            return new UnauthorizedResponse<>(null);

        if (coAuthor == null) return new ResponseEntity<>(HttpStatus.NOT_FOUND);

       if (!researchesServ.deleteCoAuthor(research, coAuthor))
           return new ResponseEntity<>(HttpStatus.BAD_REQUEST);

        return new ResponseEntity<>(HttpStatus.OK);
    }


    ///////
    //views part
    @PostMapping("/addView/{url}")
    public ResponseEntity<ResearchDTO> addView(@PathVariable String url){
        Research research = researchesServ.getResearchByUrl(url);
        if (research ==null) return new ResponseEntity<>(HttpStatus.NOT_FOUND);

        return new ResponseEntity<>(ResearchDTO.construct(researchesServ.addView(research)), HttpStatus.OK);
    }

}
